--- asterisk-1.4.10.1/apps/app_read.c~	2006-09-07 12:54:36.000000000 -0600
+++ asterisk-1.4.10.1/apps/app_read.c	2007-10-15 19:44:59.000000000 -0600
@@ -49,12 +49,14 @@
 	OPT_SKIP = (1 << 0),
 	OPT_INDICATION = (1 << 1),
 	OPT_NOANSWER = (1 << 2),
+	OPT_USE_DEFAULT = (1 << 3),
 } read_option_flags;
 
 AST_APP_OPTIONS(read_app_options, {
 	AST_APP_OPTION('s', OPT_SKIP),
 	AST_APP_OPTION('i', OPT_INDICATION),
 	AST_APP_OPTION('n', OPT_NOANSWER),
+	AST_APP_OPTION('d', OPT_USE_DEFAULT),
 });
 
 static char *app = "Read";
@@ -71,10 +73,11 @@
 "                press the '#' key).\n"
 "                Defaults to 0 - no limit - wait for the user press the '#' key.\n"
 "                Any value below 0 means the same. Max accepted value is 255.\n"
-"  option     -- options are 's' , 'i', 'n'\n"
+"  option     -- options are 's', 'i', 'n', 'd'\n"
 "                's' to return immediately if the line is not up,\n"
-"                'i' to play  filename as an indication tone from your indications.conf\n"
-"                'n' to read digits even if the line is not up.\n"
+"                'i' to play filename as an indication tone from your indications.conf\n"
+"                'n' to read digits even if the line is not up,\n"
+"                'd' to use the current value of the variable as default on timeout.\n"
 "  attempts   -- if greater than 1, that many attempts will be made in the \n"
 "                event no data is entered.\n"
 "  timeout    -- An integer number of seconds to wait for a digit response. If greater\n"
@@ -126,11 +129,10 @@
 	}
 
 	if (!ast_strlen_zero(arglist.timeout)) {
-		to = atoi(arglist.timeout);
-		if (to <= 0)
-			to = 0;
-		else
-			to *= 1000;
+		/* Allow fractional timeouts. */
+		double to_d;
+		to_d = atof(arglist.timeout);
+		to = (to_d <= 0) ? 0 : to_d * 1000;
 	}
 
 	if (ast_strlen_zero(arglist.filename)) {
@@ -148,6 +150,16 @@
 		ast_module_user_remove(u);
 		return -1;
 	}
+	if (ast_test_flag(&flags,OPT_USE_DEFAULT)) {
+		const char *defaultvalue = pbx_builtin_getvar_helper(chan, arglist.variable);
+		if (defaultvalue) {
+			size_t len = strlen(defaultvalue) + 1;
+			if (len > sizeof(tmp)) {
+				len = sizeof(tmp);
+			}
+			memcpy(tmp, defaultvalue, len);
+		}
+	}
 	ts=NULL;
 	if (ast_test_flag(&flags,OPT_INDICATION)) {
 		if (!ast_strlen_zero(arglist.filename)) {
@@ -157,7 +169,7 @@
 	if (chan->_state != AST_STATE_UP) {
 		if (ast_test_flag(&flags,OPT_SKIP)) {
 			/* At the user's option, skip if the line is not up */
-			pbx_builtin_setvar_helper(chan, arglist.variable, "\0");
+			pbx_builtin_setvar_helper(chan, arglist.variable, tmp);
 			ast_module_user_remove(u);
 			return 0;
 		} else if (!ast_test_flag(&flags,OPT_NOANSWER)) {
@@ -176,7 +188,9 @@
 					res = ast_waitfordigit(chan, to);
 					ast_playtones_stop(chan);
 					if (res < 1) {
-						tmp[x]='\0';
+						if (x > 0 || !ast_test_flag(&flags,OPT_USE_DEFAULT)) {
+							tmp[x]='\0';
+						}
 						break;
 					}
 					tmp[x++] = res;
@@ -186,7 +200,7 @@
 					}
 				}
 			} else {
-				res = ast_app_getdata(chan, arglist.filename, tmp, maxdigits, to);
+				res = ast_app_getdata_use_default(chan, arglist.filename, tmp, maxdigits, to);
 			}
 			if (res > -1) {
 				pbx_builtin_setvar_helper(chan, arglist.variable, tmp);
--- asterisk-1.4.10.1/main/app.c~	2007-07-24 14:42:05.000000000 -0600
+++ asterisk-1.4.10.1/main/app.c	2007-10-15 19:24:51.000000000 -0600
@@ -106,10 +106,15 @@
  *      "ludicrous time" (essentially never times out) */
 int ast_app_getdata(struct ast_channel *c, char *prompt, char *s, int maxlen, int timeout)
 {
-	int res,to,fto;
-	/* XXX Merge with full version? XXX */
 	if (maxlen)
 		s[0] = '\0';
+	return ast_app_getdata_use_default(c, prompt, s, maxlen, timeout);
+}
+
+int ast_app_getdata_use_default(struct ast_channel *c, char *prompt, char *s, int maxlen, int timeout)
+{
+	int res,to,fto;
+	/* XXX Merge with full version? XXX */
 	if (prompt) {
 		res = ast_streamfile(c, prompt, c->language);
 		if (res < 0)
--- asterisk-1.4.10.1/main/channel.c~	2007-08-06 15:41:13.000000000 -0600
+++ asterisk-1.4.10.1/main/channel.c	2007-10-15 18:20:32.000000000 -0600
@@ -3284,7 +3284,9 @@
 		if (d < 0)
 			return -1;
 		if (d == 0) {
-			s[pos]='\0';
+		  /* Time is up, so don't modify the return string. */
+		  if (pos > 0)
+		    s[pos]='\0';
 			return 1;
 		}
 		if (d == 1) {
--- asterisk-1.4.10.1/include/asterisk/app.h~	2007-05-31 12:41:58.000000000 -0600
+++ asterisk-1.4.10.1/include/asterisk/app.h	2007-10-15 19:24:25.000000000 -0600
@@ -96,6 +96,9 @@
  */
 int ast_app_getdata(struct ast_channel *c, char *prompt, char *s, int maxlen, int timeout);
 
+/* ast_app_getdata_use_default is the same, except it doesn't zero S if the TIMEOUT expires. */
+int ast_app_getdata_use_default(struct ast_channel *c, char *prompt, char *s, int maxlen, int timeout);
+
 /*! \brief Full version with audiofd and controlfd.  NOTE: returns '2' on ctrlfd available, not '1' like other full functions */
 int ast_app_getdata_full(struct ast_channel *c, char *prompt, char *s, int maxlen, int timeout, int audiofd, int ctrlfd);
 
